import numpy as np
import pylab as pl
from numpy import fft
	
def fourierExtrapolation(x, n_predict, n_harm, n_param=26, line_fit=1, pablo_opt=0):
	# number of periods, same as N in my code
	n = x.size
	# number of harmonics in model
	#n_harm = 10                     
	
	# creates np.array ([0,1,2,3, ... n])
	t = np.arange(0, n)
	
	""" polyfit(X, Y, 1) method gives a straight line regression fit for:
		- values on the X axis (typically 0,1,2... for time series)
		- values on the Y axis (typically the Close price data) """
		
	# find linear trend in x
	p = np.polyfit(t, x, 1)         
	# option if to use line fitting
	if line_fit == 1:
		# [orig] detrended x using linear regression
		x_notrend = x - p[0] * t
	# or just the mean value
	else:
		# detrended x using only the mean value of x
		p_mean = np.array([0.0, np.average(x)])
		x_notrend = x - p_mean[0] * t
	
	
	# original x_freqdom
	# [orig comment] detrended x in frequency domain
	x_freqdom = fft.fft(x_notrend)
	#print("x_freqdom is:")
	#print(x_freqdom)
	# option to use corrected x_freqdom by Pablo
	if pablo_opt == 1:
		h=np.sort(x_freqdom)[-n_param]
		x_freqdom=[ x_freqdom[i] if np.absolute(x_freqdom[i])>=h else 0 for i in range(len(x_freqdom)) ]
	
	# frequencies
	f = fft.fftfreq(n)
	#print("f is:")
	#print(f)
	indexes = list(range(n))
	# sort indexes by frequency, lower -> higher
	indexes.sort(key = lambda i: np.absolute(f[i]))
	#print("indexes is:")
	#print(indexes)
 
	# create the result zero-filled at first array
	t = np.arange(0, n + n_predict)
	restored_sig = np.zeros(t.size)
	#print("restored_sig is:")
	#print(restored_sig)
	for i in indexes[:1 + n_harm * 2]:
		# amplitude
		ampli = np.absolute(x_freqdom[i]) / n
		# phase
		phase = np.angle(x_freqdom[i])          
		restored_sig += ampli * np.cos(2 * np.pi * f[i] * t + phase)
		#print("restored_sig at stage " + str(i) + ":")
		#print(restored_sig)
	
	# original return statement
	#return restored_sig + p[0] * t
	return restored_sig

	
def main(list_):
	x = np.array(list_)
	n_predict = 100
	extrapolation = fourierExtrapolation(x, n_predict)
	pl.plot(np.arange(0, extrapolation.size), extrapolation, 'r', label = 'extrapolation')
	pl.plot(np.arange(0, x.size), x, 'b', label = 'x', linewidth = 3)
	pl.legend()
	pl.show()
	
if __name__ == "__main__":
	main()